@@ -1,5 +1,6 @@
 from CommDspy.rx.slicer import slicer
-from CommDspy.rx.decoding import decoding
+from CommDspy.rx.demapping import demapping
+from CommDspy.rx.decoding import decoding_gray, decoding_differential, decoding_manchester, decoding_bipolar, decoding_mlt3, decoding_differential_manchester
 from CommDspy.rx.symbol2bin import symbol2bin
 from CommDspy.rx.lock_pattern import lock_pattern_to_signal_binary, lock_pattern_to_signal
 from CommDspy.rx.checker import prbs_checker as prbs_checker
